@@ -1,28 +1,14 @@
-#
 # Copyright (C) 2013-2014 Red Hat, Inc.
 # Copyright (C) 2013 Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import datetime
 import glob
+import io
 import logging
 import os
-import StringIO
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
@@ -79,14 +65,14 @@ class vmmSnapshotPage(vmmGObjectUI):
             "on_snapshot_apply_clicked": self._on_apply_clicked,
             "on_snapshot_list_changed": self._snapshot_selected,
             "on_snapshot_list_button_press_event": self._popup_snapshot_menu,
-            "on_snapshot_refresh_clicked": self._refresh_snapshots,
+            "on_snapshot_refresh_clicked": self._on_refresh_clicked,
             "on_snapshot_list_row_activated": self._on_start_clicked,
 
             # 'Create' dialog
             "on_snapshot_new_delete_event": self._snapshot_new_close,
             "on_snapshot_new_ok_clicked": self._on_new_ok_clicked,
-            "on_snapshot_new_cancel_clicked" : self._snapshot_new_close,
-            "on_snapshot_new_name_changed" : self._snapshot_new_name_changed,
+            "on_snapshot_new_cancel_clicked": self._snapshot_new_close,
+            "on_snapshot_new_name_changed": self._snapshot_new_name_changed,
             "on_snapshot_new_name_activate": self._on_new_ok_clicked,
         })
 
@@ -106,6 +92,7 @@ class vmmSnapshotPage(vmmGObjectUI):
 
         self._snapshot_new.destroy()
         self._snapshot_new = None
+        self._snapmenu = None
 
     def _init_ui(self):
         # pylint: disable=redefined-variable-type
@@ -192,7 +179,7 @@ class vmmSnapshotPage(vmmGObjectUI):
                 for snap in self.vm.list_snapshots():
                     if name == snap.get_name():
                         snaps.append(snap)
-            except:
+            except Exception:
                 pass
 
         snaps = []
@@ -222,7 +209,7 @@ class vmmSnapshotPage(vmmGObjectUI):
 
         try:
             snapshots = self.vm.list_snapshots()
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
             self._set_error_page(_("Error refreshing snapshot list: %s") %
                                 str(e))
@@ -233,7 +220,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         for snap in snapshots:
             desc = snap.get_xmlobj().description
             name = snap.get_name()
-            state = util.xml_escape(snap.run_status())
+            state = snap.run_status()
             if snap.is_external():
                 has_external = True
                 sortname = "3%s" % name
@@ -244,7 +231,8 @@ class vmmSnapshotPage(vmmGObjectUI):
                 sortname = "1%s" % name
 
             label = "%s\n<span size='small'>%s: %s%s</span>" % (
-                (name, _("VM State"), state, external))
+                (util.xml_escape(name), _("VM State"),
+                 util.xml_escape(state), external))
             model.append([name, label, desc, snap.run_status_icon_name(),
                           sortname, snap.is_current()])
 
@@ -303,7 +291,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         mime = _mime_to_ext(os.path.splitext(filename)[1][1:], reverse=True)
         if not mime:
             return
-        return self._make_screenshot_pixbuf(mime, file(filename, "rb").read())
+        return self._make_screenshot_pixbuf(mime, open(filename, "rb").read())
 
     def _set_snapshot_state(self, snap=None):
         self.widget("snapshot-notebook").set_current_page(0)
@@ -393,7 +381,7 @@ class vmmSnapshotPage(vmmGObjectUI):
             flags = 0
             mime = self.vm.get_backend().screenshot(stream, screen, flags)
 
-            ret = StringIO.StringIO()
+            ret = io.BytesIO()
             def _write_cb(_stream, data, userdata):
                 ignore = stream
                 ignore = userdata
@@ -405,14 +393,14 @@ class vmmSnapshotPage(vmmGObjectUI):
             try:
                 if stream:
                     stream.finish()
-            except:
+            except Exception:
                 pass
 
     def _get_screenshot(self):
         if not self.vm.is_active():
             logging.debug("Skipping screenshot since VM is not active")
             return
-        if not self.vm.get_graphics_devices():
+        if not self.vm.xmlobj.devices.graphics:
             logging.debug("Skipping screenshot since VM has no graphics")
             return
 
@@ -423,7 +411,7 @@ class vmmSnapshotPage(vmmGObjectUI):
             # https://bugs.launchpad.net/qemu/+bug/1314293
             self._take_screenshot()
             mime, sdata = self._take_screenshot()
-        except:
+        except Exception:
             logging.exception("Error taking screenshot")
             return
 
@@ -460,9 +448,7 @@ class vmmSnapshotPage(vmmGObjectUI):
         self.widget("snapshot-new-ok").set_sensitive(bool(src.get_text()))
 
     def _new_finish_cb(self, error, details, newname):
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-                Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         if error is not None:
             error = _("Error creating snapshot: %s") % error
@@ -473,16 +459,16 @@ class vmmSnapshotPage(vmmGObjectUI):
     def _validate_new_snapshot(self):
         name = self.widget("snapshot-new-name").get_text()
         desc = self.widget("snapshot-new-description"
-            ).get_buffer().get_property("text")
+                           ).get_buffer().get_property("text")
 
         try:
             newsnap = DomainSnapshot(self.vm.conn.get_backend())
             newsnap.name = name
             newsnap.description = desc or None
             newsnap.validate()
-            newsnap.get_xml_config()
+            newsnap.get_xml()
             return newsnap
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Error validating snapshot: %s") % e)
 
     def _get_screenshot_data_for_save(self):
@@ -508,7 +494,7 @@ class vmmSnapshotPage(vmmGObjectUI):
             basesn = os.path.join(cachedir, "snap-screenshot-%s" % name)
 
             # Remove any pre-existing screenshots so we don't show stale data
-            for ext in mimemap.values():
+            for ext in list(mimemap.values()):
                 p = basesn + "." + ext
                 if os.path.exists(basesn + "." + ext):
                     os.unlink(p)
@@ -518,8 +504,8 @@ class vmmSnapshotPage(vmmGObjectUI):
 
             filename = basesn + "." + _mime_to_ext(mime)
             logging.debug("Writing screenshot to %s", filename)
-            file(filename, "wb").write(sndata)
-        except:
+            open(filename, "wb").write(sndata)
+        except Exception:
             logging.exception("Error saving screenshot")
 
     def _create_new_snapshot(self):
@@ -527,15 +513,12 @@ class vmmSnapshotPage(vmmGObjectUI):
         if not snap:
             return
 
-        xml = snap.get_xml_config()
+        xml = snap.get_xml()
         name = snap.name
         mime, sndata = self._get_screenshot_data_for_save()
-
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-                Gdk.Cursor.new(Gdk.CursorType.WATCH))
-
         self._snapshot_new_close()
+
+        self.set_finish_cursor()
         progWin = vmmAsyncJob(
                     self._do_create_snapshot, [xml, name, mime, sndata],
                     self._new_finish_cb, [name],
@@ -554,9 +537,9 @@ class vmmSnapshotPage(vmmGObjectUI):
         desc = desc_widget.get_buffer().get_property("text") or ""
 
         xmlobj = snap.get_xmlobj()
-        origxml = xmlobj.get_xml_config()
+        origxml = xmlobj.get_xml()
         xmlobj.description = desc
-        newxml = xmlobj.get_xml_config()
+        newxml = xmlobj.get_xml()
 
         self.vm.log_redefine_xml_diff(snap, origxml, newxml)
         if newxml == origxml:
@@ -607,6 +590,9 @@ class vmmSnapshotPage(vmmGObjectUI):
         self._snapshot_new.show()
         self.widget("snapshot-new-name").grab_focus()
 
+    def _on_refresh_clicked(self, ignore):
+        self._refresh_snapshots()
+
     def _on_start_clicked(self, ignore, ignore2=None, ignore3=None):
         snaps = self._get_selected_snapshots()
         if not snaps or len(snaps) > 1:
@@ -668,6 +654,6 @@ class vmmSnapshotPage(vmmGObjectUI):
 
         try:
             self._set_snapshot_state(snap[0])
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
             self._set_error_page(_("Error selecting snapshot: %s") % str(e))
