@@ -1,28 +1,12 @@
-#
 # Copyright (C) 2008, 2013 Red Hat, Inc.
 # Copyright (C) 2008 Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import logging
 import time
 
-from gi.repository import GObject
-
 from virtinst import pollhelpers
 from virtinst import StoragePool, StorageVolume
 from virtinst import util
@@ -111,7 +95,7 @@ class vmmStorageVolume(vmmLibvirtObject):
 
 class vmmStoragePool(vmmLibvirtObject):
     __gsignals__ = {
-        "refreshed": (GObject.SignalFlags.RUN_FIRST, None, [])
+        "refreshed": (vmmLibvirtObject.RUN_FIRST, None, [])
     }
 
     def __init__(self, conn, backend, key):
@@ -161,6 +145,10 @@ class vmmStoragePool(vmmLibvirtObject):
         vmmLibvirtObject._invalidate_xml(self)
         self._volumes = None
 
+    def _cleanup(self):
+        vmmLibvirtObject._cleanup(self)
+        self._volumes = None
+
 
     ###########
     # Actions #
