@@ -1,31 +1,25 @@
 #
 # Copyright (C) 2013, 2014 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 #
 
 """
 Configuration variables that can be set at build time
 """
 
-import ConfigParser
 import os
+import sys
 
+if sys.version_info.major != 3 or sys.version_info.minor < 4:
+    print("python 3.4 or later is required, your's is %s" %
+            sys.version_info)
+    sys.exit(1)
 
-_cfg = ConfigParser.ConfigParser()
+import configparser
+
+_cfg = configparser.ConfigParser()
 _filepath = os.path.abspath(__file__)
 _srcdir = os.path.abspath(os.path.join(os.path.dirname(_filepath), ".."))
 _cfgpath = os.path.join(os.path.dirname(_filepath), "cli.cfg")
@@ -46,7 +40,7 @@ def _get_param(name, default):
         return default
     try:
         return _cfg.get("config", name)
-    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+    except (configparser.NoOptionError, configparser.NoSectionError):
         return default
 
 
@@ -68,7 +62,7 @@ def _setup_gsettings_path(schemadir):
         raise RuntimeError("Failed to compile local gsettings schemas")
 
 
-__version__ = "1.4.3"
+__version__ = "2.1.0"
 
 
 class _CLIConfig(object):
@@ -76,14 +70,6 @@ class _CLIConfig(object):
         self.cfgpath = _cfgpath
         self.version = __version__
 
-        self.default_qemu_user = _get_param("default_qemu_user", "root")
-        self.stable_defaults = bool(int(_get_param("stable_defaults", "0")))
-
-        self.preferred_distros = _split_list(
-            _get_param("preferred_distros", ""))
-        self.hv_packages = _split_list(_get_param("hv_packages", ""))
-        self.askpass_package = _split_list(_get_param("askpass_packages", ""))
-        self.libvirt_packages = _split_list(_get_param("libvirt_packages", ""))
         self.default_graphics = _get_param("default_graphics", "spice")
         self.default_hvs = _split_list(_get_param("default_hvs", ""))
 
