@@ -3,26 +3,14 @@
 #
 # Copyright 2013 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import logging
 import os
 import re
 import stat
-import statvfs
+import subprocess
 
 import libvirt
 
@@ -55,7 +43,7 @@ def _lookup_vol_by_path(conn, path):
         vol = conn.storageVolLookupByPath(path)
         vol.info()
         return vol, None
-    except libvirt.libvirtError, e:
+    except libvirt.libvirtError as e:
         if (hasattr(libvirt, "VIR_ERR_NO_STORAGE_VOL") and
             e.get_error_code() != libvirt.VIR_ERR_NO_STORAGE_VOL):
             raise
@@ -89,7 +77,7 @@ def _stat_disk(path):
             # os.SEEK_END is not present on all systems
             size = os.lseek(fd, 0, 2)
             os.close(fd)
-        except:
+        except Exception:
             size = 0
         return False, size
     elif stat.S_ISREG(mode):
@@ -98,7 +86,7 @@ def _stat_disk(path):
     return True, 0
 
 
-def check_if_path_managed(conn, path):
+def _check_if_path_managed(conn, path):
     """
     Try to lookup storage objects for the passed path.
 
@@ -113,7 +101,7 @@ def check_if_path_managed(conn, path):
         return None, None
 
     # We have the parent pool, but didn't find a volume on first lookup
-    # attempt. Refresh the pool and try again, incase we were just out
+    # attempt. Refresh the pool and try again, in case we were just out
     # of date.
     try:
         pool.refresh(0)
@@ -121,16 +109,16 @@ def check_if_path_managed(conn, path):
         if verr:
             try:
                 vol = _lookup_vol_by_basename(pool, path)
-            except:
+            except Exception:
                 pass
-    except Exception, e:
+    except Exception as e:
         vol = None
         pool = None
         verr = str(e)
 
     if not vol and not pool and verr:
         raise ValueError(_("Cannot use storage %(path)s: %(err)s") %
-            {'path' : path, 'err' : verr})
+            {'path': path, 'err': verr})
 
     return vol, pool
 
@@ -157,9 +145,9 @@ def manage_path(conn, path):
     if not path:
         return None, None
 
-    if not path_is_url(path):
+    if not path_is_url(path) and not path_is_network_vol(conn, path):
         path = os.path.abspath(path)
-    vol, pool = check_if_path_managed(conn, path)
+    vol, pool = _check_if_path_managed(conn, path)
     if vol or pool or not _can_auto_manage(path):
         return vol, pool
 
@@ -175,7 +163,6 @@ def manage_path(conn, path):
     poolxml.type = poolxml.TYPE_DIR
     poolxml.target_path = dirname
     pool = poolxml.install(build=False, create=True, autostart=True)
-    conn.clear_cache(pools=True)
 
     vol = _lookup_vol_by_basename(pool, path)
     return vol, pool
@@ -187,7 +174,20 @@ def path_is_url(path):
     """
     if not path:
         return False
-    return bool(re.match("[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path))
+    return bool(re.match(r"[a-zA-Z]+(\+[a-zA-Z]+)?://.*", path))
+
+
+def path_is_network_vol(conn, path):
+    """
+    Detect if path is a network volume such as rbd, gluster, etc
+    """
+    if not path:
+        return False
+
+    for volxml in conn.fetch_all_vols():
+        if volxml.target_path == path:
+            return volxml.type == "network"
+    return False
 
 
 def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
@@ -236,13 +236,150 @@ def _get_dev_type(path, vol_xml, vol_object, pool_xml, remote):
     return "file"
 
 
+def path_definitely_exists(conn, path):
+    """
+    Return True if the path certainly exists, False if we are unsure.
+    See DeviceDisk entry point for more details
+    """
+    if path is None:
+        return False
+
+    try:
+        (vol, pool) = _check_if_path_managed(conn, path)
+        ignore = pool
+        if vol:
+            return True
+
+        if not conn.is_remote():
+            return os.path.exists(path)
+    except Exception:
+        pass
+
+    return False
+
+
+#########################
+# ACL/path perm helpers #
+#########################
+
+def _fix_perms_acl(dirname, username):
+    cmd = ["setfacl", "--modify", "user:%s:x" % username, dirname]
+    proc = subprocess.Popen(cmd,
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
+    out, err = proc.communicate()
+
+    logging.debug("Ran command '%s'", cmd)
+    if out or err:
+        logging.debug("out=%s\nerr=%s", out, err)
+
+    if proc.returncode != 0:
+        raise ValueError(err)
+
+
+def _fix_perms_chmod(dirname):
+    logging.debug("Setting +x on %s", dirname)
+    mode = os.stat(dirname).st_mode
+    newmode = mode | stat.S_IXOTH
+    os.chmod(dirname, newmode)
+    if os.stat(dirname).st_mode != newmode:
+        # Trying to change perms on vfat at least doesn't work
+        # but also doesn't seem to error. Try and detect that
+        raise ValueError(_("Permissions on '%s' did not stick") %
+                         dirname)
+
+
+def set_dirs_searchable(dirlist, username):
+    useacl = True
+    errdict = {}
+    for dirname in dirlist:
+        if useacl:
+            try:
+                _fix_perms_acl(dirname, username)
+                continue
+            except Exception as e:
+                logging.debug("setfacl failed: %s", e)
+                logging.debug("trying chmod")
+                useacl = False
+
+        try:
+            # If we reach here, ACL setting failed, try chmod
+            _fix_perms_chmod(dirname)
+        except Exception as e:
+            errdict[dirname] = str(e)
+
+    return errdict
+
+
+def _is_dir_searchable(dirname, uid, username):
+    """
+    Check if passed directory is searchable by uid
+    """
+    if "VIRTINST_TEST_SUITE" in os.environ:
+        return True
+
+    try:
+        statinfo = os.stat(dirname)
+    except OSError:
+        return False
+
+    if uid == statinfo.st_uid:
+        flag = stat.S_IXUSR
+    elif uid == statinfo.st_gid:
+        flag = stat.S_IXGRP
+    else:
+        flag = stat.S_IXOTH
+
+    if bool(statinfo.st_mode & flag):
+        return True
+
+    # Check POSIX ACL (since that is what we use to 'fix' access)
+    cmd = ["getfacl", dirname]
+    try:
+        proc = subprocess.Popen(cmd,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        out, err = proc.communicate()
+    except OSError:
+        logging.debug("Didn't find the getfacl command.")
+        return False
+
+    if proc.returncode != 0:
+        logging.debug("Cmd '%s' failed: %s", cmd, err)
+        return False
+
+    pattern = "user:%s:..x" % username
+    return bool(re.search(pattern.encode("utf-8", "replace"), out))
+
+
+def is_path_searchable(path, uid, username):
+    """
+    Check each dir component of the passed path, see if they are
+    searchable by the uid/username, and return a list of paths
+    which aren't searchable
+    """
+    if os.path.isdir(path):
+        dirname = path
+        base = "-"
+    else:
+        dirname, base = os.path.split(path)
+
+    fixlist = []
+    while base:
+        if not _is_dir_searchable(dirname, uid, username):
+            fixlist.append(dirname)
+        dirname, base = os.path.split(dirname)
+
+    return fixlist
+
+
 ##############################################
 # Classes for tracking storage media details #
 ##############################################
 
 class _StorageBase(object):
     """
-    Storage base class, defining the API used by VirtualDisk
+    Storage base class, defining the API used by DeviceDisk
     """
     def __init__(self, conn):
         self._conn = conn
@@ -345,16 +482,17 @@ class _StorageCreator(_StorageBase):
 
         if self._vol_install:
             self._vol_install.validate()
-        else:
-            if self._size is None:
-                raise ValueError(_("size is required for non-existent disk "
-                                   "'%s'" % self.get_path()))
+            return
+
+        if self._size is None:
+            raise ValueError(_("size is required for non-existent disk "
+                               "'%s'" % self.get_path()))
 
         err, msg = self.is_size_conflict()
         if err:
             raise ValueError(msg)
         if msg:
-            logging.warn(msg)
+            logging.warning(msg)
 
     def will_create_storage(self):
         return True
@@ -387,9 +525,12 @@ class CloneStorageCreator(_StorageCreator):
     def is_size_conflict(self):
         ret = False
         msg = None
-        vfs = os.statvfs(os.path.dirname(self._path))
-        avail = vfs[statvfs.F_FRSIZE] * vfs[statvfs.F_BAVAIL]
-        need = long(self._size * 1024L * 1024L * 1024L)
+        if self.get_dev_type() == "block":
+            avail = _stat_disk(self._path)[1]
+        else:
+            vfs = os.statvfs(os.path.dirname(self._path))
+            avail = vfs.f_frsize * vfs.f_bavail
+        need = int(self._size) * 1024 * 1024 * 1024
         if need > avail:
             if self._sparse:
                 msg = _("The filesystem will not have enough free space"
@@ -402,15 +543,15 @@ class CloneStorageCreator(_StorageCreator):
 
             if msg:
                 msg += (_(" %d M requested > %d M available") %
-                        ((need / (1024 * 1024)), (avail / (1024 * 1024))))
+                        ((need // (1024 * 1024)), (avail // (1024 * 1024))))
         return (ret, msg)
 
     def create(self, progresscb):
         text = (_("Cloning %(srcfile)s") %
-                {'srcfile' : os.path.basename(self._input_path)})
+                {'srcfile': os.path.basename(self._input_path)})
 
-        size_bytes = long(self.get_size() * 1024L * 1024L * 1024L)
-        progresscb.start(filename=self._output_path, size=long(size_bytes),
+        size_bytes = int(self.get_size()) * 1024 * 1024 * 1024
+        progresscb.start(filename=self._output_path, size=size_bytes,
                          text=text)
 
         # Plain file clone
@@ -434,7 +575,8 @@ class CloneStorageCreator(_StorageCreator):
             sparse = True
             fd = None
             try:
-                fd = os.open(self._output_path, os.O_WRONLY | os.O_CREAT, 0640)
+                fd = os.open(self._output_path, os.O_WRONLY | os.O_CREAT,
+                             0o640)
                 os.ftruncate(fd, size_bytes)
             finally:
                 if fd:
@@ -454,7 +596,7 @@ class CloneStorageCreator(_StorageCreator):
             try:
                 src_fd = os.open(self._input_path, os.O_RDONLY)
                 dst_fd = os.open(self._output_path,
-                                 os.O_WRONLY | os.O_CREAT, 0640)
+                                 os.O_WRONLY | os.O_CREAT, 0o640)
 
                 i = 0
                 while 1:
@@ -474,7 +616,7 @@ class CloneStorageCreator(_StorageCreator):
                     i += s
                     if i < size_bytes:
                         meter.update(i)
-            except OSError, e:
+            except OSError as e:
                 raise RuntimeError(_("Error cloning diskimage %s to %s: %s") %
                                 (self._input_path, self._output_path, str(e)))
         finally:
@@ -488,7 +630,7 @@ class ManagedStorageCreator(_StorageCreator):
     """
     Handles storage creation via libvirt APIs. All the actual creation
     logic lives in StorageVolume, this is mostly about pulling out bits
-    from that class and mapping them to VirtualDisk elements
+    from that class and mapping them to DeviceDisk elements
     """
     def __init__(self, conn, vol_install):
         _StorageCreator.__init__(self, conn)
@@ -544,6 +686,7 @@ class StorageBackend(_StorageBase):
         if self._vol_xml is None:
             self._vol_xml = StorageVolume(self._conn,
                 parsexml=self._vol_object.XMLDesc(0))
+            self._vol_xml.pool = self._parent_pool
         return self._vol_xml
 
     def get_parent_pool(self):
