@@ -1,21 +1,8 @@
 #
 # Copyright 2013 Red Hat, Inc.
-# Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 """
 Classes for building and installing libvirt <network> XML
 """
@@ -29,20 +16,20 @@ from .xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 
 
 class _NetworkDHCPRange(XMLBuilder):
-    _XML_ROOT_NAME = "range"
+    XML_NAME = "range"
     start = XMLProperty("./@start")
     end = XMLProperty("./@end")
 
 
 class _NetworkDHCPHost(XMLBuilder):
-    _XML_ROOT_NAME = "host"
+    XML_NAME = "host"
     macaddr = XMLProperty("./@mac")
     name = XMLProperty("./@name")
     ip = XMLProperty("./@ip")
 
 
 class _NetworkIP(XMLBuilder):
-    _XML_ROOT_NAME = "ip"
+    XML_NAME = "ip"
 
     family = XMLProperty("./@family")
     address = XMLProperty("./@address")
@@ -56,14 +43,9 @@ class _NetworkIP(XMLBuilder):
     ranges = XMLChildProperty(_NetworkDHCPRange, relative_xpath="./dhcp")
     hosts = XMLChildProperty(_NetworkDHCPHost, relative_xpath="./dhcp")
 
-    def add_range(self):
-        r = _NetworkDHCPRange(self.conn)
-        self.add_child(r)
-        return r
-
 
 class _NetworkRoute(XMLBuilder):
-    _XML_ROOT_NAME = "route"
+    XML_NAME = "route"
 
     family = XMLProperty("./@family")
     address = XMLProperty("./@address")
@@ -72,18 +54,35 @@ class _NetworkRoute(XMLBuilder):
     netmask = XMLProperty("./@netmask")
 
 
+class _NetworkForwardPf(XMLBuilder):
+    XML_NAME = "pf"
+    dev = XMLProperty("./@dev")
+
+
+class _NetworkForwardAddress(XMLBuilder):
+    XML_NAME = "address"
+    type = XMLProperty("./@type")
+    domain = XMLProperty("./@domain", is_int=True)
+    bus = XMLProperty("./@bus", is_int=True)
+    slot = XMLProperty("./@slot", is_int=True)
+    function = XMLProperty("./@function", is_int=True)
+
+
 class _NetworkForward(XMLBuilder):
-    _XML_ROOT_NAME = "forward"
+    XML_NAME = "forward"
 
     mode = XMLProperty("./@mode")
     dev = XMLProperty("./@dev")
+    managed = XMLProperty("./@managed")
+    pf = XMLChildProperty(_NetworkForwardPf)
+    vfs = XMLChildProperty(_NetworkForwardAddress)
 
     def pretty_desc(self):
         return Network.pretty_forward_desc(self.mode, self.dev)
 
 
 class _NetworkBandwidth(XMLBuilder):
-    _XML_ROOT_NAME = "bandwidth"
+    XML_NAME = "bandwidth"
 
     inbound_average = XMLProperty("./inbound/@average")
     inbound_peak = XMLProperty("./inbound/@peak")
@@ -135,7 +134,7 @@ class _NetworkBandwidth(XMLBuilder):
 
 
 class _NetworkPortgroup(XMLBuilder):
-    _XML_ROOT_NAME = "portgroup"
+    XML_NAME = "portgroup"
 
     name = XMLProperty("./@name")
     default = XMLProperty("./@default", is_yesno=True)
@@ -161,7 +160,7 @@ class Network(XMLBuilder):
             else:
                 if dev:
                     desc = (_("%(mode)s to %(device)s") %
-                            {"mode" : mode, "device" : dev})
+                            {"mode": mode, "device": dev})
                 else:
                     desc = _("%s network") % mode.capitalize()
         else:
@@ -169,10 +168,6 @@ class Network(XMLBuilder):
 
         return desc
 
-    def __init__(self, *args, **kwargs):
-        XMLBuilder.__init__(self, *args, **kwargs)
-        self._random_uuid = None
-
 
     ###################
     # Helper routines #
@@ -191,36 +186,32 @@ class Network(XMLBuilder):
     # Validation helpers #
     ######################
 
-    def _validate_name(self, name):
+    @staticmethod
+    def validate_name(conn, name):
         util.validate_name(_("Network"), name)
 
         try:
-            self.conn.networkLookupByName(name)
+            conn.networkLookupByName(name)
         except libvirt.libvirtError:
             return
         raise ValueError(_("Name '%s' already in use by another network." %
                          name))
 
-    def _get_default_uuid(self):
-        if self._random_uuid is None:
-            self._random_uuid = util.generate_uuid(self.conn)
-        return self._random_uuid
-
 
     ##################
     # XML properties #
     ##################
 
-    _XML_ROOT_NAME = "network"
-    _XML_PROP_ORDER = ["ipv6", "name", "uuid", "forward",
+    XML_NAME = "network"
+    _XML_PROP_ORDER = ["ipv6", "name", "uuid", "forward", "virtualport_type",
                        "bridge", "stp", "delay", "domain_name",
                        "macaddr", "ips", "routes", "bandwidth"]
 
     ipv6 = XMLProperty("./@ipv6", is_yesno=True)
-    name = XMLProperty("./name", validate_cb=_validate_name)
-    uuid = XMLProperty("./uuid",
-                       validate_cb=lambda s, v: util.validate_uuid(v),
-                       default_cb=_get_default_uuid)
+    name = XMLProperty("./name")
+    uuid = XMLProperty("./uuid")
+
+    virtualport_type = XMLProperty("./virtualport/@type")
 
     # Not entirely correct, there can be multiple routes
     forward = XMLChildProperty(_NetworkForward, is_single=True)
@@ -237,22 +228,13 @@ class Network(XMLBuilder):
     routes = XMLChildProperty(_NetworkRoute)
     bandwidth = XMLChildProperty(_NetworkBandwidth, is_single=True)
 
-    def add_ip(self):
-        ip = _NetworkIP(self.conn)
-        self.add_child(ip)
-        return ip
-    def add_route(self):
-        route = _NetworkRoute(self.conn)
-        self.add_child(route)
-        return route
-
 
     ##################
     # build routines #
     ##################
 
     def install(self, start=True, autostart=True):
-        xml = self.get_xml_config()
+        xml = self.get_xml()
         logging.debug("Creating virtual network '%s' with xml:\n%s",
                       self.name, xml)
 
@@ -262,7 +244,7 @@ class Network(XMLBuilder):
                 net.create()
             if autostart:
                 net.setAutostart(autostart)
-        except:
+        except Exception:
             net.undefine()
             raise
 
